#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <linux/fb.h>
#include <sys/mman.h>
#include <sys/ioctl.h>

#define SQUARE_SIZE 256

int main()
{
    int fbfd = 0;
    struct fb_var_screeninfo vinfo;
    struct fb_fix_screeninfo finfo;
    long int screensize = 0;
    char *fbp = 0;
    int x = 0, y = 0, x_offs = 0, y_offs = 0;
    long int location = 0;

    // Open the file for reading and writing
    fbfd = open("/dev/fb0", O_RDWR);
    if (fbfd == -1) {
        perror("Error: cannot open framebuffer device");
        exit(1);
    }
    printf("The framebuffer device was opened successfully.\n");

    // Get fixed screen information
    if (ioctl(fbfd, FBIOGET_FSCREENINFO, &finfo) == -1) {
        perror("Error reading fixed information");
        exit(2);
    }

    // Get variable screen information
    if (ioctl(fbfd, FBIOGET_VSCREENINFO, &vinfo) == -1) {
        perror("Error reading variable information");
        exit(3);
    }

    printf("%dx%d, %dbpp\n", vinfo.xres, vinfo.yres, vinfo.bits_per_pixel);

    // Figure out the size of the screen in bytes
    screensize = vinfo.xres * vinfo.yres * vinfo.bits_per_pixel / 8;

    // Map the device to memory
    fbp = (char *)mmap(0, screensize, PROT_READ | PROT_WRITE, MAP_SHARED,
                       fbfd, 0);
    if ((int)fbp == -1) {
        perror("Error: failed to map framebuffer device to memory");
        exit(4);
    }
    printf("The framebuffer device was mapped to memory successfully.\n");

    // Set Display bg-color
    memset(fbp, 0xff, screensize);

    // Place the square in the center of the screen
    x_offs = (vinfo.xres - SQUARE_SIZE) / 2;
    y_offs = (vinfo.yres - SQUARE_SIZE) / 2;

    // Figure out where in memory to put the pixel
    for (y = 0; y < SQUARE_SIZE; y++)
        for (x = 0; x < SQUARE_SIZE; x++) {

            location = (x+x_offs+vinfo.xoffset) * (vinfo.bits_per_pixel/8) +
                       (y+y_offs+vinfo.yoffset) * finfo.line_length;

            if (vinfo.bits_per_pixel == 32) {
                *(fbp + location + 0) = 0xeb * (SQUARE_SIZE - y) / SQUARE_SIZE;
                *(fbp + location + 1) = 0x98 * (SQUARE_SIZE - y) / SQUARE_SIZE;
                *(fbp + location + 2) = 0x15 * (SQUARE_SIZE - y) / SQUARE_SIZE;
                *(fbp + location + 3) = 0x00;      // No transparency
            } else  { //assume 16bpp (565)
                unsigned int r = (0x15>>3) * (SQUARE_SIZE - y) / SQUARE_SIZE;
                unsigned int g = (0x98>>2) * (SQUARE_SIZE - y) / SQUARE_SIZE;
                unsigned int b = (0xeb>>3) * (SQUARE_SIZE - y) / SQUARE_SIZE;
                unsigned short int t = r << 11 | g << 5 | b << 0;
                *((unsigned short int*)(fbp + location)) = t;
            }

        }
    munmap(fbp, screensize);
    close(fbfd);
    return 0;
}
